package com.qiaoba.common.database.config;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.qiaoba.common.database.constants.DynamicDatasourceConstant;
import com.qiaoba.common.database.properties.DefaultDataSourceProperties;
import com.qiaoba.common.database.properties.MasterInfo;
import com.qiaoba.common.database.properties.PoolInfo;
import com.qiaoba.common.database.properties.SlaveInfo;
import com.qiaoba.common.database.utils.JdbcUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 多数据源配置
 *
 * @author ailanyin
 * @version 1.0
 * @since 2021/10/15 0015 下午 16:43
 */
@Slf4j
@Configuration
public class DynamicDataSourceConfig {

    @Resource
    private DynamicDataSourceContext dataSource;
    @Resource
    private DefaultDataSourceProperties defaultDataSourceProperties;

    public static Map<Object, Object> DATA_SOURCE_MAP = new ConcurrentHashMap<>();

    /**
     * 把DynamicDataSourceContext 纳入容器管理，其他地方使用DynamicDataSourceConfig 类可以直接从容器取对象，并调用freshDataSource方法
     */
    @PostConstruct
    public void init() {
        MasterInfo master = defaultDataSourceProperties.getMaster();
        List<SlaveInfo> slaves = defaultDataSourceProperties.getSlaves();
        addDataSourceToMap(DynamicDatasourceConstant.DEFAULT_MASTER_DATASOURCE_KEY, buildDataSource(master.getDriver(), master.getUrl(), master.getUsername(), master.getPassword(), master.getPool()));
        if (CollUtil.isNotEmpty(slaves)) {
            for (int i = 0; i < slaves.size(); i++) {
                SlaveInfo slave = slaves.get(i);
                if (slave.getIsUse()) {
                    addDataSourceToMap(DynamicDatasourceConstant.DEFAULT_SLAVE_DATASOURCE_PREFIX + i, buildDataSource(slave.getDriver(), slave.getUrl(), slave.getUsername(), slave.getPassword(), slave.getPool()));
                }
            }
        }
        dataSource.freshDataSource(DATA_SOURCE_MAP);
    }

    private static Object buildDataSource(String driver, String url, String username, String password, PoolInfo poolInfo) {
        boolean flag = JdbcUtil.checkConnect(driver, url, username, password);
        if (!flag) {
            log.error("数据源: " + url + " , 连接失败，请检查！");
            return null;
        }
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setUrl(url);
        dataSource.setUsername(username);
        dataSource.setPassword(password);
        dataSource.setDriverClassName(driver);
        dataSource.setInitialSize(poolInfo.getInit());
        dataSource.setMinIdle(poolInfo.getMin());
        dataSource.setMaxActive(poolInfo.getMax());
        try {
            dataSource.init();
            return dataSource;
        } catch (SQLException e) {
            dataSource.close();
            return null;
        }
    }

    /**
     * 程序关闭后,要释放数据源连接池
     */
    @PreDestroy
    public void close() {
        Set<Map.Entry<Object, Object>> entries = DATA_SOURCE_MAP.entrySet();
        for (Map.Entry<Object, Object> entry : entries) {
            DruidDataSource dataSource = (DruidDataSource) entry.getValue();
            dataSource.close();
        }
    }

    private void addDataSourceToMap(String name, Object dataSource) {
        if (Objects.nonNull(dataSource)) {
            DATA_SOURCE_MAP.put(name, dataSource);
        }
    }
}
