package com.qiaoba.common.redis.service.impl;


import com.qiaoba.common.base.context.BaseContext;
import com.qiaoba.common.redis.service.RedisService;
import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 自定义Redis接口实现类
 *
 * @author ailanyin
 * @version 1.0
 * @since 2021/10/2 0003 下午 17:44
 */
@Service
@RequiredArgsConstructor
public class RedisServiceImpl implements RedisService {

    private final RedisTemplate<String, Object> redisTemplate;

    @Override
    public void set(String key, Object value, long time) {
        redisTemplate.opsForValue().set(handleKey(key), value, time, TimeUnit.SECONDS);
    }

    @Override
    public void set(String key, Object value) {
        redisTemplate.opsForValue().set(handleKey(key), value);
    }

    @Override
    public Object get(String key) {
        return redisTemplate.opsForValue().get(handleKey(key));
    }

    @Override
    public Boolean del(String key) {
        return redisTemplate.delete(handleKey(key));
    }

    @Override
    public void del(Collection<String> keys) {
        List<String> list = keys.stream().map(key -> key = handleKey(key)).collect(Collectors.toList());
        redisTemplate.delete(list);
    }

    @Override
    public Boolean expire(String key, long time) {
        return redisTemplate.expire(key, time, TimeUnit.SECONDS);
    }

    @Override
    public Long getExpire(String key) {
        return redisTemplate.getExpire(handleKey(key), TimeUnit.SECONDS);
    }

    @Override
    public Boolean hasKey(String key) {
        return redisTemplate.hasKey(handleKey(key));
    }

    @Override
    public Long incr(String key, long delta) {
        return redisTemplate.opsForValue().increment(handleKey(key), delta);
    }

    @Override
    public Long decr(String key, long delta) {
        return redisTemplate.opsForValue().increment(handleKey(key), -delta);
    }

    @Override
    public Object hGet(String key, String hashKey) {
        return redisTemplate.opsForHash().get(handleKey(key), hashKey);
    }

    @Override
    public Boolean hSet(String key, String hashKey, Object value, long time) {
        String newKey = handleKey(key);
        redisTemplate.opsForHash().put(newKey, hashKey, value);
        return expire(newKey, time);
    }

    @Override
    public void hSet(String key, String hashKey, Object value) {
        redisTemplate.opsForHash().put(handleKey(key), hashKey, value);
    }

    @Override
    public Map<Object, Object> hGetAll(String key) {
        return redisTemplate.opsForHash().entries(handleKey(key));
    }

    @Override
    public Boolean hSetAll(String key, Map<String, Object> map, long time) {
        String newKey = handleKey(key);
        redisTemplate.opsForHash().putAll(newKey, map);
        return expire(newKey, time);
    }

    @Override
    public void hSetAll(String key, Map<String, ?> map) {
        redisTemplate.opsForHash().putAll(handleKey(key), map);
    }

    @Override
    public void hDel(String key, Object... hashKey) {
        redisTemplate.opsForHash().delete(handleKey(key), hashKey);
    }

    @Override
    public Boolean hHasKey(String key, String hashKey) {
        return redisTemplate.opsForHash().hasKey(handleKey(key), hashKey);
    }

    @Override
    public Long hIncr(String key, String hashKey, Long delta) {
        return redisTemplate.opsForHash().increment(handleKey(key), hashKey, delta);
    }

    @Override
    public Long hDecr(String key, String hashKey, Long delta) {
        return redisTemplate.opsForHash().increment(handleKey(key), hashKey, -delta);
    }

    @Override
    public Set<Object> sMembers(String key) {
        return redisTemplate.opsForSet().members(handleKey(key));
    }

    @Override
    public Long sAdd(String key, Object... values) {
        return redisTemplate.opsForSet().add(handleKey(key), values);
    }

    @Override
    public Long sAdd(String key, long time, Object... values) {
        String newKey = handleKey(key);
        Long count = redisTemplate.opsForSet().add(newKey, values);
        expire(newKey, time);
        return count;
    }

    @Override
    public Boolean sIsMember(String key, Object value) {
        return redisTemplate.opsForSet().isMember(handleKey(key), value);
    }

    @Override
    public Long sSize(String key) {
        return redisTemplate.opsForSet().size(handleKey(key));
    }

    @Override
    public Long sRemove(String key, Object... values) {
        return redisTemplate.opsForSet().remove(handleKey(key), values);
    }

    @Override
    public List<Object> lRange(String key, long start, long end) {
        return redisTemplate.opsForList().range(handleKey(key), start, end);
    }

    @Override
    public Long lSize(String key) {
        return redisTemplate.opsForList().size(handleKey(key));
    }

    @Override
    public Object lIndex(String key, long index) {
        return redisTemplate.opsForList().index(handleKey(key), index);
    }

    @Override
    public Long lPush(String key, Object value) {
        return redisTemplate.opsForList().rightPush(handleKey(key), value);
    }

    @Override
    public Long lPush(String key, Object value, long time) {
        String newKey = handleKey(key);
        Long index = redisTemplate.opsForList().rightPush(newKey, value);
        expire(newKey, time);
        return index;
    }

    @Override
    public Long lPushAll(String key, Object... values) {
        return redisTemplate.opsForList().rightPushAll(handleKey(key), values);
    }

    @Override
    public Long lPushAll(String key, Long time, Object... values) {
        String newKey = handleKey(key);
        Long count = redisTemplate.opsForList().rightPushAll(newKey, values);
        expire(newKey, time);
        return count;
    }

    @Override
    public Long lRemove(String key, long count, Object value) {
        return redisTemplate.opsForList().remove(handleKey(key), count, value);
    }


    @Override
    public Collection<String> getKeys(String key) {
        return redisTemplate.keys(handleKey(key));
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> T getObject(String key, Class<T> clazz) {
        return (T) get(key);
    }

    @Override
    @SuppressWarnings("unchecked")
    public <T> List<T> getObjectList(String key, Class<T> clazz) {
        return (List<T>) get(key);
    }

    private String handleKey(String key) {
        StringBuilder sb = new StringBuilder();
        sb.append("tenant_").append(BaseContext.getTenantId()).append(":").append(key);
        return sb.toString();
    }
}
