package com.qiaoba.module.system.service.impl;

import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.LineCaptcha;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.qiaoba.api.system.entity.SysUser;
import com.qiaoba.api.system.entity.dto.LoginDto;
import com.qiaoba.api.system.service.SysUserDetailsApiService;
import com.qiaoba.auth.constants.SecurityConstant;
import com.qiaoba.auth.utils.SecurityUtil;
import com.qiaoba.auth.utils.TokenUtil;
import com.qiaoba.common.base.enums.BaseEnum;
import com.qiaoba.common.base.exceptions.ServiceException;
import com.qiaoba.common.redis.service.RedisService;
import com.qiaoba.module.system.service.SysLoginService;
import com.qiaoba.module.system.service.SysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * 登录 服务层实现
 *
 * @author ailanyin
 * @version 1.0
 * @since 2023/5/15 15:31
 */
@Service
@RequiredArgsConstructor
public class SysLoginServiceImpl implements SysLoginService {

    private final RedisService redisService;
    private final HttpServletRequest request;
    private final SysUserDetailsApiService userDetailsService;
    private final SysUserService sysUserService;

    @Override
    public Map<String, Object> getCaptchaImage() {
        Map<String, Object> map = new HashMap<String, Object>(4);

        map.put("register", getRegisterConfig());
        if (!getCaptchaConfig()) {
            map.put("captchaEnabled", false);
            return map;
        }

        String uuid = UUID.randomUUID().toString(true);
        LineCaptcha captcha = CaptchaUtil.createLineCaptcha(150, 50, 4, 20);
        map.put("uuid", uuid);
        map.put("img", captcha.getImageBase64());
        redisService.set(SecurityConstant.CAPTCHA_KEY + uuid, captcha.getCode(), 120);

        return map;
    }

    @Override
    public String login(LoginDto dto) {
        // 校验验证码
        validateCaptcha(dto.getCode(), dto.getUuid());
        // username查询用户信息
        SysUser sysUser = sysUserService.selectByUsername(dto.getUsername());
        // 检查账号信息
        validateUser(dto.getUsername(), sysUser);
        // 检验密码
        validatePassword(dto.getUsername(), sysUser.getPassword(), dto.getPassword());
        // 缓存user
        userDetailsService.toCache(sysUser.getUsername());
        // 生成Token
        return TokenUtil.generateToken(sysUser.getUsername());
    }

    private void validatePassword(String username, String password, String inputPassword) {
        boolean result = SecurityUtil.matchesPassword(inputPassword, password);
        if (!result) {
            ThreadUtil.execAsync(() -> beforePasswordError(username));
            throw new ServiceException(SecurityConstant.PASSWORD_ERROR);
        }
    }

    private void beforePasswordError(String username) {

    }

    private void validateUser(String username, SysUser user) {
        if (ObjectUtil.isNull(user)) {
            throw new ServiceException(StrUtil.format("登录用户：{} 不存在", username));
        } else if (BaseEnum.ABNORMAL.getCode().equals(user.getStatus())) {
            throw new ServiceException(StrUtil.format("对不起, 您的账号：{} 已被禁用", username));
        }
    }

    private boolean getCaptchaConfig() {
        return true;
/*        try {
            return SecurityConstant.CAPTCHA_ON.equals(redisService.get(SecurityConstant.CAPTCHA_ON_OFF_KEY));
        } catch (Exception e) {
            throw new ServiceException("Redis中验证码配置不存在！");
        }*/
    }

    private boolean getRegisterConfig() {
        return true;
       /* try {
            return SecurityConstant.REGISTER_ON.equals(redisService.get(SecurityConstant.REGISTER_ON_OFF_KEY));
        } catch (Exception e) {
            throw new ServiceException("Redis中注册配置不存在！");
        }*/
    }

    private void validateCaptcha(String code, String uuid) {
        if (getCaptchaConfig()) {
            if (StrUtil.isBlank(code) || StrUtil.isBlank(uuid)) {
                throw new ServiceException("验证码或uuid获取失败！");
            }
            try {
                if (!redisService.hasKey(SecurityConstant.CAPTCHA_KEY + uuid)) {
                    throw new ServiceException("验证码已经过期失效！");
                } else {
                    if (!code.equalsIgnoreCase(redisService.get(SecurityConstant.CAPTCHA_KEY + uuid).toString())) {
                        throw new ServiceException("验证码输入错误！");
                    }
                }

            } finally {
                redisService.del(SecurityConstant.CAPTCHA_KEY + uuid);
            }
        }
    }
}
